@@ -9,9 +9,8 @@ use std::{
 };
 
 use anyhow::{anyhow, bail};
-use git_repository::{
-    actor, bstr::BString, interrupt, objs, odb, odb::pack, prelude::*, progress, refs::file::ReferenceExt, Progress,
-};
+use git_repository as git;
+use git_repository::{actor, bstr::BString, interrupt, objs, prelude::*, progress, refs::file::ReferenceExt, Progress};
 use itertools::Itertools;
 use rayon::prelude::*;
 
@@ -46,66 +45,67 @@ where
     W: io::Write,
     P: Progress,
 {
-    let repo = git_repository::discover(working_dir)?;
-    let packed = repo.refs.packed_buffer()?;
+    let repo = git::discover(working_dir)?.apply_environment();
     let commit_id = repo
         .refs
-        .find(refname.to_string_lossy().as_ref(), packed.as_ref())?
-        .peel_to_id_in_place(&repo.refs, packed.as_ref(), |oid, buf| {
-            repo.odb
-                .try_find(oid, buf, &mut pack::cache::Never)
+        .find(refname.to_string_lossy().as_ref())?
+        .peel_to_id_in_place(&repo.refs, |oid, buf| {
+            repo.objects
+                .try_find(oid, buf)
                 .map(|obj| obj.map(|obj| (obj.kind, obj.data)))
         })?
         .to_owned();
 
-    let all_commits = {
+    let (all_commits, is_shallow) = {
         let start = Instant::now();
         let mut progress = progress.add_child("Traverse commit graph");
         progress.init(None, progress::count("commits"));
-        let mut pack_cache = odb::pack::cache::Never;
         let mut commits: Vec<Vec<u8>> = Vec::new();
-        for c in interrupt::Iter::new(
+        let commit_iter = interrupt::Iter::new(
             commit_id.ancestors(|oid, buf| {
                 progress.inc();
-                repo.odb.find(oid, buf, &mut pack_cache).ok().map(|o| {
+                repo.objects.find(oid, buf).map(|o| {
                     commits.push(o.data.to_owned());
                     objs::CommitRefIter::from_bytes(o.data)
                 })
             }),
             || anyhow!("Cancelled by user"),
-        ) {
-            c??;
+        );
+        let mut is_shallow = false;
+        for c in commit_iter {
+            match c? {
+                Ok(c) => c,
+                Err(git::traverse::commit::ancestors::Error::FindExisting { .. }) => {
+                    is_shallow = true;
+                    break;
+                }
+                Err(err) => return Err(err.into()),
+            };
         }
         progress.show_throughput(start);
-        commits
+        (commits, is_shallow)
     };
 
+    let mailmap = repo.load_mailmap();
     let start = Instant::now();
     #[allow(clippy::redundant_closure)]
     let mut all_commits: Vec<actor::Signature> = all_commits
         .into_par_iter()
-        .map(|commit_data: Vec<u8>| {
+        .filter_map(|commit_data: Vec<u8>| {
             objs::CommitRefIter::from_bytes(&commit_data)
-                .signatures()
-                .next()
-                .map(|author| actor::Signature::from(author))
+                .author()
+                .map(|author| mailmap.resolve(author.trim()))
+                .ok()
         })
-        .try_fold(
-            || Vec::new(),
-            |mut out: Vec<_>, item| {
-                out.push(item?);
-                Some(out)
-            },
+        .collect::<Vec<_>>();
+    all_commits.sort_by(|a, b| {
+        a.email.cmp(&b.email).then(
+            a.time
+                .seconds_since_unix_epoch
+                .cmp(&b.time.seconds_since_unix_epoch)
+                .reverse(),
         )
-        .try_reduce(
-            || Vec::new(),
-            |mut out, vec| {
-                out.extend(vec.into_iter());
-                Some(out)
-            },
-        )
-        .ok_or_else(|| anyhow!("An error occurred when decoding commits - one commit could not be parsed"))?;
-    all_commits.sort_by(|a, b| a.email.cmp(&b.email).then(a.time.time.cmp(&b.time.time).reverse()));
+    });
     if all_commits.is_empty() {
         bail!("No commits to process");
     }
@@ -156,10 +156,11 @@ where
         .expect("at least one commit at this point");
     writeln!(
         out,
-        "total hours: {:.02}\ntotal 8h days: {:.02}\ntotal commits = {}\ntotal authors: {}",
+        "total hours: {:.02}\ntotal 8h days: {:.02}\ntotal commits = {}{}\ntotal authors: {}",
         total_hours,
         total_hours / HOURS_PER_WORKDAY,
         total_commits,
+        is_shallow.then(|| " (shallow)").unwrap_or_default(),
         num_authors
     )?;
     if !omit_unify_identities {
@@ -186,7 +187,8 @@ fn estimate_hours(commits: &[actor::Signature]) -> WorkByEmail {
         + commits.iter().rev().tuple_windows().fold(
             0_f32,
             |hours, (cur, next): (&actor::Signature, &actor::Signature)| {
-                let change_in_minutes = (next.time.time - cur.time.time) as f32 / MINUTES_PER_HOUR;
+                let change_in_minutes =
+                    (next.time.seconds_since_unix_epoch - cur.time.seconds_since_unix_epoch) as f32 / MINUTES_PER_HOUR;
                 if change_in_minutes < MAX_COMMIT_DIFFERENCE_IN_MINUTES {
                     hours + change_in_minutes as f32 / MINUTES_PER_HOUR
                 } else {
