@@ -1,4 +1,7 @@
-use std::path::{Path, PathBuf};
+use std::{
+    ffi::OsStr,
+    path::{Path, PathBuf},
+};
 
 use git_config::file::GitConfig;
 use git_repository::{objs::bstr::ByteSlice, progress, Progress};
@@ -23,19 +26,27 @@ enum RepoKind {
 fn find_git_repository_workdirs<P: Progress>(
     root: impl AsRef<Path>,
     mut progress: P,
+    debug: bool,
 ) -> impl Iterator<Item = (PathBuf, RepoKind)>
 where
     <P as Progress>::SubProgress: Sync,
 {
     progress.init(None, progress::count("filesystem items"));
     fn is_repository(path: &Path) -> Option<git_repository::Kind> {
-        if !(path.is_dir() && path.ends_with(".git")) {
+        // Can be git dir or worktree checkout (file)
+        if path.file_name() != Some(OsStr::new(".git")) {
             return None;
         }
-        if path.join("HEAD").is_file() && path.join("config").is_file() {
-            git_repository::path::is::git(path).ok()
+
+        if path.is_dir() {
+            if path.join("HEAD").is_file() && path.join("config").is_file() {
+                git_repository::path::is::git(path).ok()
+            } else {
+                None
+            }
         } else {
-            None
+            // git files are always worktrees
+            Some(git_repository::Kind::WorkTree)
         }
     }
     fn into_workdir(git_dir: PathBuf) -> PathBuf {
@@ -64,7 +75,10 @@ where
     #[cfg(all(target_os = "macos", target_arch = "aarch64"))]
     let walk = walk.parallelism(jwalk::Parallelism::RayonNewPool(4));
 
-    walk.process_read_dir(move |_depth, _path, _read_dir_state, siblings| {
+    walk.process_read_dir(move |_depth, path, _read_dir_state, siblings| {
+        if debug {
+            eprintln!("{}", path.display());
+        }
         let mut found_any_repo = false;
         let mut found_bare_repo = false;
         for entry in siblings.iter_mut().flatten() {
@@ -74,7 +88,7 @@ where
                 entry.client_state = State { is_repo: true, is_bare };
                 entry.read_children_path = None;
 
-                found_any_repo = !is_bare;
+                found_any_repo = true;
                 found_bare_repo = is_bare;
             }
         }
@@ -177,7 +191,11 @@ fn handle(
             match kind {
                 RepoKind::Bare => path,
                 RepoKind::WorkingTree => {
-                    path.set_extension("");
+                    if let Some(ext) = path.extension() {
+                        if ext == "git" {
+                            path.set_extension("");
+                        }
+                    }
                     path
                 }
             }
@@ -208,11 +226,14 @@ pub fn discover<P: Progress>(
     source_dir: impl AsRef<Path>,
     mut out: impl std::io::Write,
     mut progress: P,
+    debug: bool,
 ) -> anyhow::Result<()>
 where
     <<P as Progress>::SubProgress as Progress>::SubProgress: Sync,
 {
-    for (git_workdir, _kind) in find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories")) {
+    for (git_workdir, _kind) in
+        find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories"), debug)
+    {
         writeln!(&mut out, "{}", git_workdir.display())?;
     }
     Ok(())
@@ -229,7 +250,9 @@ where
 {
     let mut num_errors = 0usize;
     let destination = destination.as_ref().canonicalize()?;
-    for (path_to_move, kind) in find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories")) {
+    for (path_to_move, kind) in
+        find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories"), false)
+    {
         if let Err(err) = handle(mode, kind, &path_to_move, &destination, &mut progress) {
             progress.fail(format!(
                 "Error when handling directory {:?}: {}",
