@@ -14,6 +14,7 @@ use git_repository::{
         transport,
         transport::client::Capabilities,
     },
+    Progress,
 };
 
 use crate::{remote::refs::JsonRef, OutputFormat};
@@ -25,6 +26,7 @@ pub struct Context<W> {
     pub format: OutputFormat,
     pub should_interrupt: Arc<AtomicBool>,
     pub out: W,
+    pub object_hash: git_repository::hash::Kind,
 }
 
 struct CloneDelegate<W> {
@@ -302,7 +304,7 @@ fn print(out: &mut impl io::Write, res: pack::bundle::write::Outcome, refs: &[Re
 fn write_raw_refs(refs: &[Ref], directory: PathBuf) -> std::io::Result<()> {
     let assure_dir_exists = |path: &BString| {
         assert!(!path.starts_with_str("/"), "no ref start with a /, they are relative");
-        let path = directory.join(path.to_path().expect("UTF-8 conversion succeeds"));
+        let path = directory.join(git_features::path::from_byte_slice_or_panic_on_windows(path));
         std::fs::create_dir_all(path.parent().expect("multi-component path")).map(|_| path)
     };
     for r in refs {
@@ -322,13 +324,14 @@ fn receive_pack_blocking<W: io::Write>(
     mut refs_directory: Option<PathBuf>,
     ctx: &mut Context<W>,
     input: impl io::BufRead,
-    progress: impl git_repository::Progress,
+    progress: impl Progress,
     refs: &[Ref],
 ) -> io::Result<()> {
     let options = pack::bundle::write::Options {
         thread_limit: ctx.thread_limit,
         index_kind: pack::index::Version::V2,
         iteration_mode: pack::data::input::Mode::Verify,
+        object_hash: ctx.object_hash,
     };
     let outcome =
         pack::Bundle::write_to_directory(input, directory.take(), progress, &ctx.should_interrupt, None, options)
